@@ -30,7 +30,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.55";
+$VERSION = "0.56";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -68,7 +68,6 @@ my %def_attr = (
     verbatim		=> 0,
     types		=> undef,
 
-
     _EOF		=> 0,
     _STATUS		=> undef,
     _FIELDS		=> undef,
@@ -126,10 +125,11 @@ my %_cache_id = (	# Keep in sync with XS!
 sub _set_attr_C
 {
     my ($self, $name, $val) = @_;
+    defined $val or $val = 0;
     $self->{$name} = $val;
     $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
-    $cache[$_cache_id{$name}] = defined $val ? unpack "C", $val : 0;
+    $cache[$_cache_id{$name}] = unpack "C", $val;
     $self->{_CACHE} = pack "C*", @cache;
     } # _set_attr_C
 
@@ -137,10 +137,9 @@ sub _set_attr_N
 {
     my ($self, $name, $val) = @_;
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
     my @cache = unpack "C*", $self->{_CACHE};
     my $i = $_cache_id{$name};
-    $cache[$i++] = $_ for unpack "C*", pack "N", defined $val ? $val : 0;
+    $cache[$i++] = $_ for unpack "C*", pack "N", $val;
     $self->{_CACHE} = pack "C*", @cache;
     } # _set_attr_N
 
@@ -172,6 +171,7 @@ sub eol
     my $self = shift;
     if (@_) {
 	my $eol = shift;
+	defined $eol or $eol = "";
 	my $eol_len = length $eol;
 	$self->{eol} = $eol;
 	$self->{_CACHE} or return;
@@ -530,7 +530,7 @@ perhaps better called ASV (anything separated values) rather than just CSV.
 =head2 Embedded newlines
 
 B<Important Note>: The default behavior is to only accept ascii characters.
-This means that fields can not contain newlines. If your data contains 
+This means that fields can not contain newlines. If your data contains
 newlines embedded in fields, or characters above 0x7e (tilde), or binary data,
 you *must* set C<< binary => 1 >> in the call to C<new ()>.  To cover the widest
 range of parsing options, you will always want to set binary.
@@ -558,7 +558,7 @@ or, more safely in perl 5.6 and up
  open my $io, "<", $file or die "$file: $!";
  while (my $row = $csv->getline ($io)) {
      my @fields = @$row;
- 
+
 =head2 Unicode (UTF8)
 
 On parsing (both for C<getline ()> and C<parse ()>), if the source is
@@ -568,6 +568,22 @@ are marked binary will also be marked UTF8.
 On combining (C<print ()> and C<combine ()>), if any of the combining
 fields was marked UTF8, the resulting string will be marked UTF8.
 
+For complete control over encoding, please use Text::CSV::Encoded:
+
+    use Text::CSV::Encoded;
+    my $csv = Text::CSV::Encoded->new ({
+        encoding_in  => "iso-8859-1", # the encoding comes into   Perl
+        encoding_out => "cp1252",     # the encoding comes out of Perl
+        });
+
+    $csv = Text::CSV::Encoded->new ({ encoding  => "utf8" });
+    # combine () and print () accept *literally* utf8 encoded data
+    # parse () and getline () return *literally* utf8 encoded data
+
+    $csv = Text::CSV::Encoded->new ({ encoding  => undef }); # default
+    # combine () and print () accept UTF8 marked data
+    # parse () and getline () return UTF8 marked data
+
 =head1 SPECIFICATION
 
 While no formal specification for CSV exists, RFC 4180 1) describes a common
@@ -666,9 +682,10 @@ Currently the following attributes are available:
 
 =item eol
 
-An end-of-line string to add to rows, usually C<undef> (nothing,
-default = C<$\>), C<"\012"> (Line Feed) or C<"\015\012"> (Carriage
-Return, Line Feed). Cannot be longer than 7 (ASCII) characters.
+An end-of-line string to add to rows. C<undef> is replaced with an
+empty string. The default is C<$\>. Common values for C<eol> are
+C<"\012"> (Line Feed) or C<"\015\012"> (Carriage Return, Line Feed).
+Cannot be longer than 7 (ASCII) characters.
 
 If both C<$/> and C<eol> equal C<"\015">, parsing lines that end on
 only a Carriage Return without Line Feed, will be C<parse>d correct.
@@ -773,7 +790,7 @@ doubling the quote mark in a field escapes it:
   "foo","bar","Escape ""quote mark"" with two ""quote marks""","baz"
 
 If you change the default quote_char without changing the default
-escape_char, the escape_char will still be the quote mark.  If instead 
+escape_char, the escape_char will still be the quote mark.  If instead
 you want to escape the quote_char by doubling it, you will need to change
 the escape_char to be the same as what you changed the quote_char to.
 
@@ -976,7 +993,7 @@ methods are meaningless, again.
 
 The C<getline_hr ()> and C<column_names ()> methods work together to allow
 you to have rows returned as hashrefs. You must call C<column_names ()>
-first to declare your column names. 
+first to declare your column names.
 
  $csv->column_names (qw( code name price description ));
  $hr = $csv->getline_hr ($io);
@@ -1312,17 +1329,6 @@ Using C<getline ()> and C<print ()> instead is the prefered way to go.
 We probably need many more tests to check if all edge-cases are covered.
 See t/50_utf8.t.
 
-Probably the best way to do this more reliable is to make a subclass
-Text::CSV_XS::Encoded that can be passed the required encoding and
-then behaves transparently (but slower), something like this:
-
-    use Text::CSV::Encoded;
-    my $csv = Text::CSV::Encoded->new ({
-        encoding     => "utf-8",      # Both in and out
-        encoding_in  => "iso-8859-1", # Only the input
-        encoding_out => "cp1252",     # Only the output
-        });
-
 =item Parse the whole file at once
 
 Implement a new methods that enables the parsing of a complete file
@@ -1365,11 +1371,6 @@ No guarantees, but this is what I have in mind right now:
 
 =item next + 1
 
- - allow_double_quoted
- - Text::CSV_XS::Encoded (maybe)
-
-=item next + 2
-
  - csv2csv - a script to regenerate a CSV file to follow standards
  - EBCDIC support
 
@@ -1521,8 +1522,8 @@ exhausted before the quote is found, that field is not terminated.
 =head1 SEE ALSO
 
 L<perl(1)>, L<IO::File(3)>, L<IO::Handle(3)>, L<IO::Wrap(3)>,
-L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Separator(3)>,
-and L<Spreadsheet::Read(3)>.
+L<Text::CSV(3)>, L<Text::CSV_PP(3)>, L<Text::CSV::Encoded>,
+L<Text::CSV::Separator(3)>, and L<Spreadsheet::Read(3)>.
 
 =head1 AUTHORS and MAINTAINERS
 
